@@ -1,7 +1,7 @@
 /*
  * The SIP module interface.
  *
- * Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -54,8 +54,8 @@ extern "C" {
 /*
  * Define the SIP version number.
  */
-#define SIP_VERSION         0x040c02
-#define SIP_VERSION_STR     "4.12.2"
+#define SIP_VERSION         0x040e01
+#define SIP_VERSION_STR     "4.14.1"
 
 
 /*
@@ -68,6 +68,17 @@ extern "C" {
  *
  * History:
  *
+ * 9.1  Added the capsule type.
+ *      Added the 'z' format character to sip_api_build_result().
+ *      Added the 'z', '!' and '$' format characters to
+ *      sip_api_parse_result_ex().
+ *
+ * 9.0  Changed the sipVariableGetterFunc signature.
+ *      Added sip_api_parse_result_ex() to the private API.
+ *      Added sip_api_call_error_handler() to the private API.
+ *      Added em_virterrorhandlers to sipExportedModuleDef.
+ *      Re-ordered the API functions.
+ *
  * 8.1  Revised the sipVariableDef structure.
  *      sip_api_get_address() is now part of the public API.
  *
@@ -174,7 +185,7 @@ extern "C" {
  *
  * 0.0  Original version.
  */
-#define SIP_API_MAJOR_NR    8
+#define SIP_API_MAJOR_NR    9
 #define SIP_API_MINOR_NR    1
 
 
@@ -192,12 +203,6 @@ extern "C" {
 typedef unsigned int uint;
 
 
-/* Some compatibility stuff to help with handwritten code for SIP v3. */
-#if !defined(ANY)
-#define ANY     void
-#endif
-
-
 /* Some Python compatibility stuff. */
 #if PY_VERSION_HEX >= 0x02050000
 
@@ -268,6 +273,15 @@ typedef unsigned int uint;
 #endif
 
 
+#if defined(SIP_USE_PYCAPSULE)
+#define SIPCapsule_FromVoidPtr(p, n)    PyCapsule_New((p), (n), NULL)
+#define SIPCapsule_AsVoidPtr(p, n)      PyCapsule_GetPointer((p), (n))
+#else
+#define SIPCapsule_FromVoidPtr(p, n)    sipConvertFromVoidPtr((p))
+#define SIPCapsule_AsVoidPtr(p, n)      sipConvertToVoidPtr((p))
+#endif
+
+
 /*
  * The mask that can be passed to sipTrace().
  */
@@ -329,14 +343,16 @@ typedef void *(*sipCastFunc)(void *, const struct _sipTypeDef *);
 typedef const struct _sipTypeDef *(*sipSubClassConvertFunc)(void **);
 typedef int (*sipConvertToFunc)(PyObject *, void **, int *, PyObject *);
 typedef PyObject *(*sipConvertFromFunc)(void *, PyObject *);
-typedef int (*sipVirtHandlerFunc)(void *, PyObject *, ...);
+typedef void (*sipVirtErrorHandlerFunc)(struct _sipSimpleWrapper *);
+typedef int (*sipVirtHandlerFunc)(sip_gilstate_t, sipVirtErrorHandlerFunc,
+        struct _sipSimpleWrapper *, PyObject *, ...);
 typedef void (*sipAssignFunc)(void *, SIP_SSIZE_T, const void *);
 typedef void *(*sipArrayFunc)(SIP_SSIZE_T);
 typedef void *(*sipCopyFunc)(const void *, SIP_SSIZE_T);
 typedef void (*sipReleaseFunc)(void *, int);
 typedef PyObject *(*sipPickleFunc)(void *);
 typedef int (*sipAttrGetterFunc)(const struct _sipTypeDef *, PyObject *);
-typedef PyObject *(*sipVariableGetterFunc)(void *, PyObject *);
+typedef PyObject *(*sipVariableGetterFunc)(void *, PyObject *, PyObject *);
 typedef int (*sipVariableSetterFunc)(void *, PyObject *, PyObject *);
 
 
@@ -686,25 +702,6 @@ typedef struct _sipVariableDef {
 } sipVariableDef;
 
 
-/*
- * The information describing a variable.  This is deprecated from v8.1 of the
- * API and should be removed in v9.0.
- */
-typedef struct _sipVariableDef_8 {
-    /* The variable name. */
-    const char *vd_name;
-
-    /* The variable getter. */
-    sipVariableGetterFunc vd_getter;
-
-    /* The variable setter.  It is NULL if the variable is const. */
-    sipVariableSetterFunc vd_setter;
-
-    /* This is set if the variable is static. */
-    int vd_is_static;
-} sipVariableDef_8;
-
-
 /*
  * The information describing a type, either a C++ class (or C struct), a C++
  * namespace, a mapped type or a named enum.
@@ -1037,6 +1034,9 @@ typedef struct _sipExportedModuleDef {
     /* The table of virtual handlers. */
     sipVirtHandlerFunc *em_virthandlers;
 
+    /* The table of virtual error handlers. */
+    sipVirtErrorHandlerFunc *em_virterrorhandlers;
+
     /* The sub-class convertors. */
     sipSubClassConvertorDef *em_convertors;
 
@@ -1372,6 +1372,7 @@ typedef struct _sipAPIDef {
             sipAttrGetterFunc getter);
     int (*api_is_api_enabled)(const char *name, int from, int to);
     sipErrorState (*api_bad_callable_arg)(int arg_nr, PyObject *arg);
+    void *(*api_get_address)(struct _sipSimpleWrapper *w);
 
     /*
      * The following are deprecated parts of the public API.
@@ -1450,10 +1451,11 @@ typedef struct _sipAPIDef {
             PyObject *sipKwdArgs, const char **kwdlist, PyObject **unused,
             const char *fmt, ...);
     void (*api_add_exception)(sipErrorState es, PyObject **parseErrp);
-    /*
-     * The following are part of the public API.
-     */
-    void *(*api_get_address)(struct _sipSimpleWrapper *w);
+    int (*api_parse_result_ex)(sip_gilstate_t, sipVirtErrorHandlerFunc,
+            sipSimpleWrapper *, PyObject *method, PyObject *res,
+            const char *fmt, ...);
+    void (*api_call_error_handler)(sipVirtErrorHandlerFunc,
+            sipSimpleWrapper *);
 } sipAPIDef;
 
 
@@ -1506,13 +1508,15 @@ typedef struct _sipQtAPI {
 /*
  * Useful macros, not part of the public API.
  */
-#define SIP_PY_OWNED        0x0004  /* Owned by Python. */
+#define SIP_PY_OWNED        0x0004  /* If owned by Python. */
 #define SIP_INDIRECT        0x0008  /* If there is a level of indirection. */
 #define SIP_ACCFUNC         0x0010  /* If there is an access function. */
-#define SIP_NOT_IN_MAP      0x0020  /* If Python object not in the map. */
+#define SIP_NOT_IN_MAP      0x0020  /* If Python object is not in the map. */
 #define SIP_SHARE_MAP       0x0040  /* If the map slot might be occupied. */
 #define SIP_CPP_HAS_REF     0x0080  /* If C/C++ has a reference. */
 #define SIP_POSSIBLE_PROXY  0x0100  /* If there might be a proxy slot. */
+#define SIP_ALIAS           0x0200  /* If it is an alias. */
+#define SIP_CREATED         0x0400  /* If the C/C++ object has been created. */
 
 #define sipIsPyOwned(w)     ((w)->flags & SIP_PY_OWNED)
 #define sipSetPyOwned(w)    ((w)->flags |= SIP_PY_OWNED)
@@ -1527,6 +1531,8 @@ typedef struct _sipQtAPI {
 #define sipResetCppHasRef(w)    ((w)->flags &= ~SIP_CPP_HAS_REF)
 #define sipPossibleProxy(w) ((w)->flags & SIP_POSSIBLE_PROXY)
 #define sipSetPossibleProxy(w)  ((w)->flags |= SIP_POSSIBLE_PROXY)
+#define sipIsAlias(w)       ((w)->flags & SIP_ALIAS)
+#define sipWasCreated(w)    ((w)->flags & SIP_CREATED)
 
 
 #define SIP_TYPE_TYPE_MASK  0x0007  /* The type type mask. */
@@ -1553,9 +1559,13 @@ typedef struct _sipQtAPI {
 
 #define sipIsExactWrappedType(wt)   (sipTypeAsPyTypeObject((wt)->type) == (PyTypeObject *)(wt))
 
-#define sipConvertFromSliceObject(o,len,start,stop,step,slen) \
+#if PY_VERSION_HEX >= 0x03020000
+#define sipConvertFromSliceObject   PySlice_GetIndicesEx
+#else
+#define sipConvertFromSliceObject(o, len, start, stop, step, slen) \
         PySlice_GetIndicesEx((PySliceObject *)(o), (len), (start), (stop), \
                 (step), (slen))
+#endif
 
 
 /*
