@@ -35,13 +35,13 @@ _pkg_config = {
     'py_conf_inc_dir':    '/usr/local/include/python2.7',
     'py_inc_dir':         '/usr/local/include/python2.7',
     'py_lib_dir':         '/usr/local/lib/python2.7/config',
-    'py_version':         0x020701,
+    'py_version':         0x020705,
     'sip_bin':            '/usr/local/bin/sip',
     'sip_config_args':    '-b /usr/local/bin -d /usr/local/lib/python2.7/site-packages -e /usr/local/include/python2.7 -v /usr/local/share/sip INCDIR=/usr/local/include LIBDIR=/usr/local/lib CC=cc CXX=c++ CFLAGS=-O2 -pipe  CXXFLAGS=-O2 -pipe ',
     'sip_inc_dir':        '/usr/local/include/python2.7',
     'sip_mod_dir':        '/usr/local/lib/python2.7/site-packages',
-    'sip_version':        0x040c02,
-    'sip_version_str':    '4.12.2',
+    'sip_version':        0x040e01,
+    'sip_version_str':    '4.14.1',
     'universal':          ''
 }
 
@@ -50,6 +50,7 @@ _default_macros = {
     'AR':                       'ar q',
     'CC':                       'cc',
     'CFLAGS':                   '-O2 -pipe ',
+    'CFLAGS_APP':               '',
     'CFLAGS_CONSOLE':           '',
     'CFLAGS_DEBUG':             '-g',
     'CFLAGS_EXCEPTIONS_OFF':    '',
@@ -72,6 +73,7 @@ _default_macros = {
     'COPY':                     'cp -f',
     'CXX':                      'c++',
     'CXXFLAGS':                 '-O2 -pipe ',
+    'CXXFLAGS_APP':             '',
     'CXXFLAGS_CONSOLE':         '',
     'CXXFLAGS_DEBUG':           '-g',
     'CXXFLAGS_EXCEPTIONS_OFF':  '',
@@ -365,6 +367,7 @@ class Makefile:
         self._debug = debug
         self._makefile = makefile
         self._installs = installs
+        self._infix = ""
 
         # Make sure the destination directory is an absolute path.
         if dir:
@@ -462,12 +465,16 @@ class Makefile:
             win_exceptions = ("exceptions" in wcfg)
             win_rtti = ("rtti" in wcfg)
             win_stl = ("stl" in wcfg)
+
+            qt_version = self.config.qt_version
         else:
             win_shared = 1
             win_exceptions = 0
             win_rtti = 0
             win_stl = 0
 
+            qt_version = 0
+
         # Get what we are going to transform.
         cflags = _UniqueList()
         cflags.extend(self.extra_cflags)
@@ -527,10 +534,18 @@ class Makefile:
         rpaths = _UniqueList()
 
         for l in self.extra_lib_dirs:
+            l_dir = os.path.dirname(l)
+
+            # This is a hack to ignore PyQt's internal support libraries.
+            if '/qpy/' in l_dir:
+                continue
+
             # Ignore relative directories.  This is really a hack to handle
             # SIP v3 inter-module linking.
-            if os.path.dirname(l) not in ("", ".", ".."):
-                rpaths.append(l)
+            if l_dir in ("", ".", ".."):
+                continue
+
+            rpaths.append(l)
 
         if self._python:
             incdir.append(self.config.py_inc_dir)
@@ -648,13 +663,21 @@ class Makefile:
             lflags.extend(self.optional_list("LFLAGS_THREAD"))
 
         if self._qt:
+            # Get the name of the mkspecs directory.
+            try:
+                specd_base = self.config.qt_data_dir
+            except AttributeError:
+                specd_base = self.config.qt_dir
+
+            mkspecs = os.path.join(specd_base, "mkspecs")
+
             if self.generator != "UNIX" and win_shared:
                 defines.append("QT_DLL")
 
             if not self._debug:
                 defines.append("QT_NO_DEBUG")
 
-            if self.config.qt_version >= 0x040000:
+            if qt_version >= 0x040000:
                 for mod in self._qt:
                     # Note that qmake doesn't define anything for QtHelp.
                     if mod == "QtCore":
@@ -685,6 +708,9 @@ class Makefile:
                         defines.append("QT_XMLPATTERNS_LIB")
                     elif mod == "phonon":
                         defines.append("QT_PHONON_LIB")
+
+                    if qt_version >= 0x050000 and mod in ("QtSql", "QtTest"):
+                        defines.append("QT_WIDGETS_LIB")
             elif self._threaded:
                 defines.append("QT_THREAD_SUPPORT")
 
@@ -693,7 +719,11 @@ class Makefile:
             libdir.extend(libdir_qt)
             rpaths.extend(libdir_qt)
 
-            if self.config.qt_version >= 0x040000:
+            if qt_version >= 0x040000:
+                # Try and read QT_LIBINFIX from qconfig.pri.
+                qconfig = os.path.join(mkspecs, "qconfig.pri")
+                self._infix = self._extract_value(qconfig, "QT_LIBINFIX")
+
                 # For Windows: the macros that define the dependencies on
                 # Windows libraries.
                 wdepmap = {
@@ -738,7 +768,7 @@ class Makefile:
                 # include QtGui and QtNetwork as a dependency any longer.  This
                 # seems to be a bug in Qt v4.2.0.  We explicitly set the
                 # dependencies here.
-                if self.config.qt_version >= 0x040200 and "QtAssistant" in self._qt:
+                if qt_version >= 0x040200 and "QtAssistant" in self._qt:
                     if "QtGui" not in self._qt:
                         self._qt.append("QtGui")
                     if "QtNetwork" not in self._qt:
@@ -776,7 +806,7 @@ class Makefile:
                 qt_lib = self.config.qt_lib
 
                 if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE") and win_shared:
-                    qt_lib = qt_lib + version_to_string(self.config.qt_version).replace(".", "")
+                    qt_lib = qt_lib + version_to_string(qt_version).replace(".", "")
 
                     if self.config.qt_edition == "non-commercial":
                         qt_lib = qt_lib + "nc"
@@ -785,33 +815,41 @@ class Makefile:
                 libs.extend(self._dependent_libs(self.config.qt_lib))
 
             # Handle header directories.
-            try:
-                specd_base = self.config.qt_data_dir
-            except AttributeError:
-                specd_base = self.config.qt_dir
-
-            specd = os.path.join(specd_base, "mkspecs", "default")
+            specd = os.path.join(mkspecs, "default")
 
             if not os.access(specd, os.F_OK):
-                specd = os.path.join(specd_base, "mkspecs", self.config.platform)
+                specd = os.path.join(mkspecs, self.config.platform)
 
             incdir.append(specd)
 
             qtincdir = self.optional_list("INCDIR_QT")
 
             if qtincdir:
-                if self.config.qt_version >= 0x040000:
+                if qt_version >= 0x040000:
                     for mod in self._qt:
                         if mod == "QAxContainer":
                             incdir.append(os.path.join(qtincdir[0], "ActiveQt"))
                         elif self._is_framework(mod):
-                            if mod == "QtAssistant" and self.config.qt_version < 0x040202:
+                            if mod == "QtAssistant" and qt_version < 0x040202:
                                 mod = "QtAssistantClient"
 
-                            incdir.append(os.path.join(libdir_qt[0], mod + ".framework", "Headers"))
+                            incdir.append(os.path.join(libdir_qt[0],
+                                    mod + ".framework", "Headers"))
+
+                            if qt_version >= 0x050000 and mod == "QtGui":
+                                incdir.append(os.path.join(libdir_qt[0],
+                                        "QtWidgets.framework", "Headers"))
+                                incdir.append(os.path.join(libdir_qt[0],
+                                        "QtPrintSupport.framework", "Headers"))
                         else:
                             incdir.append(os.path.join(qtincdir[0], mod))
 
+                            if qt_version >= 0x050000 and mod == "QtGui":
+                                incdir.append(os.path.join(qtincdir[0],
+                                        "QtWidgets"))
+                                incdir.append(os.path.join(qtincdir[0],
+                                        "QtPrintSupport"))
+
                 # This must go after the module include directories.
                 incdir.extend(qtincdir)
 
@@ -822,7 +860,7 @@ class Makefile:
             libs.extend(self.optional_list("LIBS_OPENGL"))
 
         if self._qt or self._opengl:
-            if self.config.qt_version < 0x040000 or self._opengl or "QtGui" in self._qt:
+            if qt_version < 0x040000 or self._opengl or "QtGui" in self._qt:
                 incdir.extend(self.optional_list("INCDIR_X11"))
                 libdir.extend(self.optional_list("LIBDIR_X11"))
                 libs.extend(self.optional_list("LIBS_X11"))
@@ -879,10 +917,15 @@ class Makefile:
         else:
             lib = mname
 
+        lib += self._infix
+
         if self._debug:
             if sys.platform == "win32":
                 lib = lib + "d"
-            elif self.config.qt_version < 0x040200 or sys.platform == "darwin":
+            elif sys.platform == "darwin":
+                if not self._is_framework(mname):
+                    lib = lib + "_debug"
+            elif self.config.qt_version < 0x040200:
                 lib = lib + "_debug"
 
         if sys.platform == "win32" and "shared" in self.config.qt_winconfig.split():
@@ -968,8 +1011,6 @@ class Makefile:
         clib is the library name in cannonical form.
         framework is set of the library is implemented as a MacOS framework.
         """
-        prl_libs = []
-
         if self.generator in ("MSVC", "MSVC.NET", "MSBUILD", "BMAKE"):
             prl_name = os.path.join(self.config.qt_lib_dir, clib + ".prl")
         elif sys.platform == "darwin" and framework:
@@ -977,27 +1018,42 @@ class Makefile:
         else:
             prl_name = os.path.join(self.config.qt_lib_dir, "lib" + clib + ".prl")
 
-        if os.access(prl_name, os.F_OK):
+        libs = self._extract_value(prl_name, "QMAKE_PRL_LIBS").split()
+
+        if self.config.qt_version >= 0x050000 and clib == "QtGui":
+            for xtra in ("QtWidgets", "QtPrintSupport"):
+                libs.extend(self.platform_lib(xtra, framework).split())
+
+        return libs
+
+    def _extract_value(self, fname, vname):
+        """Return the stripped value from a name=value line in a file.
+
+        fname is the name of the file.
+        vname is the name of the value.
+        """
+        value = ""
+
+        if os.access(fname, os.F_OK):
             try:
-                f = open(prl_name, "r")
+                f = open(fname, "r")
             except IOError:
-                error("Unable to open \"%s\"" % prl_name)
+                error("Unable to open \"%s\"" % fname)
 
             line = f.readline()
             while line:
                 line = line.strip()
                 if line and line[0] != "#":
                     eq = line.find("=")
-                    if eq > 0 and line[:eq].strip() == "QMAKE_PRL_LIBS":
-                        prl_libs = line[eq + 1:].split()
+                    if eq > 0 and line[:eq].strip() == vname:
+                        value = line[eq + 1:].strip()
                         break
 
                 line = f.readline()
 
             f.close()
 
-        return prl_libs
-
+        return value
 
     def parse_build_file(self, filename):
         """
@@ -1853,6 +1909,9 @@ class ProgramMakefile(Makefile):
 
         build.extend(self.optional_list("CXXFLAGS"))
 
+        # This is for Qt5.
+        build.extend(self.optional_list("CXXFLAGS_APP"))
+
         # Borland requires all flags to precede all file names.
         if self.generator != "BMAKE":
             build.append(source)
@@ -2202,6 +2261,9 @@ def create_config_module(module, template, content, macros=None):
 
         line = sf.readline()
 
+    df.close()
+    sf.close()
+
 
 def version_to_sip_tag(version, tags, description):
     """Convert a version number to a SIP tag.
@@ -2363,10 +2425,13 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                 self._openfile(nextfile)
                 return self.readline()
 
-            if not line and self.filestack:
-                self.currentfile = self.filestack.pop()
-                self.path = self.pathstack.pop()
-                return self.readline()
+            if not line:
+                self.currentfile.close()
+
+                if self.filestack:
+                    self.currentfile = self.filestack.pop()
+                    self.path = self.pathstack.pop()
+                    return self.readline()
 
             return line
 
@@ -2449,7 +2514,7 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                     if orig_rhs is not None:
                         rhs = orig_rhs + " " + rhs
 
-                raw[lhs] = rhs
+                raw[lhs] = _expand_macro_value(raw, rhs, properties)
 
         line = f.readline()
 
@@ -2465,7 +2530,7 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
 
     for lhs in list(raw.keys()):
         # Strip any prefix.
-        if lhs.find(macro_prefix) == 0:
+        if lhs.startswith(macro_prefix):
             reflhs = lhs[len(macro_prefix):]
         else:
             reflhs = lhs
@@ -2476,48 +2541,6 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
 
         rhs = raw[lhs]
 
-        # Resolve any references.
-        estart = rhs.find("$$(")
-        mstart = rhs.find("$$")
-
-        while mstart >= 0 and mstart != estart:
-            rstart = mstart + 2
-            if rstart < len(rhs) and rhs[rstart] == "{":
-                rstart = rstart + 1
-                term = "}"
-            elif rstart < len(rhs) and rhs[rstart] == "[":
-                rstart = rstart + 1
-                term = "]"
-            else:
-                term = string.whitespace
-
-            mend = rstart
-            while mend < len(rhs) and rhs[mend] not in term:
-                mend = mend + 1
-
-            lhs = rhs[rstart:mend]
-
-            if term in "}]":
-                mend = mend + 1
-
-            if term == "]":
-                if properties is None or lhs not in list(properties.keys()):
-                    error("%s: property '%s' is not defined." % (filename, lhs))
-
-                value = properties[lhs]
-            else:
-                try:
-                    value = raw[lhs]
-                except KeyError:
-                    # We used to treat this as an error, but Qt v4.3.0 has at
-                    # least one case that refers to an undefined macro.  If
-                    # qmake handles it then this must be the correct behaviour.
-                    value = ""
-
-            rhs = rhs[:mstart] + value + rhs[mend:]
-            estart = rhs.find("$$(")
-            mstart = rhs.find("$$")
-
         # Expand any POSIX style environment variables.
         pleadin = ["$$(", "$("]
 
@@ -2598,6 +2621,50 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
     return refined
 
 
+def _expand_macro_value(macros, rhs, properties):
+    """Expand the value of a macro based on ones seen so far."""
+    estart = rhs.find("$$(")
+    mstart = rhs.find("$$")
+
+    while mstart >= 0 and mstart != estart:
+        rstart = mstart + 2
+        if rstart < len(rhs) and rhs[rstart] == "{":
+            rstart = rstart + 1
+            term = "}"
+        elif rstart < len(rhs) and rhs[rstart] == "[":
+            rstart = rstart + 1
+            term = "]"
+        else:
+            term = string.whitespace
+
+        mend = rstart
+        while mend < len(rhs) and rhs[mend] not in term:
+            mend = mend + 1
+
+        lhs = rhs[rstart:mend]
+
+        if term in "}]":
+            mend = mend + 1
+
+        if term == "]":
+            # Assume a missing property expands to an empty string.
+            if properties is None:
+                value = ""
+            else:
+                value = properties.get(lhs, "")
+        else:
+            # We used to treat a missing value as an error, but Qt v4.3.0 has
+            # at least one case that refers to an undefined macro.  If qmake
+            # handles it then this must be the correct behaviour.
+            value = macros.get(lhs, "")
+
+        rhs = rhs[:mstart] + value + rhs[mend:]
+        estart = rhs.find("$$(")
+        mstart = rhs.find("$$")
+
+    return rhs
+
+
 def create_wrapper(script, wrapper, gui=0, use_arch=''):
     """Create a platform dependent executable wrapper around a Python script.
 
